@@ -2,7 +2,7 @@
  *                 Mathematical Object Library                 *
  *           Conversion tools Library - Include File           *
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 2.2.0                   *
+ *	             GNU/linux version 2.4.0                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright © 2003,2004,2005,2006,2007,2008,2009 COLLARD Christophe
@@ -43,8 +43,8 @@
     \author copyright \htmlonly &#169; \endhtmlonly 2003, 2004, 2005, 2006, 2007, 2008, 2009 Christophe COLLARD \n
 	    copyright \htmlonly &#169; 2003, 2004, 2005, 2006, 2007, 2008, 2009 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
 	    copyright \htmlonly &#169; 2003, 2004, 2005, 2006, 2007 Laboratoire de Math&#233;matiques et ses Applications de Valenciennes (LAMAV) \endhtmlonly
-    \version 2.2.0
-    \date 2003-2009
+    \version 2.4.0
+    \date 2003-2010
     \bug none
     \warning none
 */
@@ -61,27 +61,27 @@
 #endif
 
 #if !defined(__PARAMETERS_H)
-#include "../parameters.h"
+#include "parameters.h"
 #endif
 
 #if !defined(__MATHS_H)
-#include "maths.h"
+#include "MOL++/maths.h"
 #endif
 
 #if !defined(__VECTORS_H)
-#include "vectors.h"
+#include "MOL++/vectors.h"
 #endif
 
 #if !defined(__MATRIX_H)
-#include "matrix.h"
+#include "MOL++/matrix.h"
 #endif
 
 #if !defined(__SYMTENSORS2_H)
-#include "symtensors2.h"
+#include "MOL++/symtensors2.h"
 #endif
 
 #if !defined(__TENSORS4_H)
-#include "tensors4.h"
+#include "MOL++/tensors4.h"
 #endif
 
 using namespace std;
