@@ -2,7 +2,7 @@
  *                 Mathematical Object Library                 *
  *	      class matrix:  declarations for matrix	       *
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 2.2.0                   *
+ *	             GNU/linux version 2.4.0                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright © 1995,1996,1999,2000,2001,2002,2003,2004,2005,2006,2007,2008,2009 COLLARD Christophe
@@ -47,8 +47,8 @@
 	     copyright \htmlonly &#169; \endhtmlonly 2003, 2004 Denis MERCIER \n
 	     copyright \htmlonly &#169; 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
 	     copyright \htmlonly &#169; 2002, 2003, 2004, 2005, 2006, 2007 Laboratoire de Math&#233;matiques et ses Applications de Valenciennes (LAMAV) \endhtmlonly
-    \version 2.2.0
-    \date 1995-2009
+    \version 2.4.0
+    \date 1995-2010
     \bug none
     \warning none
 */
@@ -78,15 +78,15 @@
 #endif
 
 #if !defined(__PARAMETERS_H)
-#include "../parameters.h"
+#include "parameters.h"
 #endif
 
 #if !defined(__MATHS_H)
-#include "maths.h"
+#include "MOL++/maths.h"
 #endif
 
 #if !defined(__VECTORS_H)
-#include "vectors.h"
+#include "MOL++/vectors.h"
 #endif
 
 using namespace std;
@@ -103,9 +103,9 @@ template <class T> class matrix
 
   private:
 
-  protected :
-  int nbRows, nbColumns;  // number of rows & columns of the matrix
-  vector<T> *mrows;
+  protected:
+    int nbRows, nbColumns;  // number of rows & columns of the matrix
+    vector<T> *mrows;
 
   public:
     matrix (); 			        // default constructor
@@ -138,28 +138,31 @@ template <class T> class matrix
     template <class Tf> friend matrix<Tf> operator / (const matrix<Tf>&, const Tf&);         // overloads scalar / matrix
     template <class Tf> friend Tf         operator | (const matrix<Tf>&, const matrix<Tf>&); // A|B = Aij Bij (for matrix & tensor2)
     template <class Tf> friend matrix<Tf> operator ^ (const vector<Tf>&, const vector<Tf>&); // overloads operator ^ for matrix = vector*t(vector) (tensorial product for 2 vectors)
-    void operator += (const matrix<T>&);  // overloads += operator for matrix
-    void operator -= (const matrix<T>&);  // overloads -= operator for matrix
-    void operator *= (const matrix<T>&);  // overloads *= operator for matrix
-    void operator *= (const T&);          // overloads *= operator between matrix and scalar
-    void operator /= (const matrix<T>&);  // overloads /= operator for matrix
-    void operator /= (const T&);	  // overloads /= operator between matrix and scalar
+    matrix<T>& operator += (const matrix<T>&);  // overloads += operator for matrix
+    matrix<T>& operator -= (const matrix<T>&);  // overloads -= operator for matrix
+    matrix<T>& operator *= (const matrix<T>&);  // overloads *= operator for matrix
+    matrix<T>& operator *= (const T&);          // overloads *= operator between matrix and scalar
+    matrix<T>& operator /= (const matrix<T>&);  // overloads /= operator for matrix
+    matrix<T>& operator /= (const T&);	        // overloads /= operator between matrix and scalar
     template <class Tf> friend bool operator == (const matrix<Tf>&, const matrix<Tf>&);  // compares 2 matrices
     template <class Tf> friend bool operator != (const matrix<Tf>&, const matrix<Tf>&);
     template <class Tf> friend ostream& operator << (ostream&, const matrix<Tf>&);       // overloads output stream for matrix
     template <class Tf> friend istream& operator >> (istream&, const matrix<Tf>&);       // overloads input stream for matrix
-    template <class Tf> friend matrix<Tf> Id2 (int);              // returns identity matrix
-    template <class Tf> friend Tf det (const matrix<Tf>&);        // computes the determinant operator for a matrix
-    template <class Tf> friend matrix<Tf> t (const matrix<Tf>&);  // defines tranposition operator for matrix
-    virtual matrix<T> inv ();	                                  // computes inversion operator for matrix
-    template <class Tf> friend matrix<Tf> gauss (matrix<Tf>);     // computes matrix inversion with gauss algorithm
-    template <class Tf> friend bool inv (matrix<Tf>&);            // same function as gauss but returns the inversion result 0/1
+    template <class Tf> friend matrix<Tf> Id2 (int);                // returns identity matrix
+    template <class Tf> friend Tf det (const matrix<Tf>&);          // computes the determinant operator for a matrix
+    template <class Tf> friend matrix<Tf> t (const matrix<Tf>&);    // defines tranposition operator for matrix
+    virtual matrix<T> inv ();	                                    // computes inversion operator for matrix
+    template <class Tf> friend matrix<Tf> gauss (matrix<Tf>);       // computes matrix inversion with gauss algorithm
+    template <class Tf> friend bool inv (matrix<Tf>&, long double); // same function as gauss but returns the inversion result 0/1
     //    friend matrix<T> gauss_pt <T> (matrix<T>);              // computes matrix inversion with gauss algorithm
-// i/o disk
+
+    // i/o disk
     virtual void save (const char*);
     virtual void save (const string&);
     virtual void read (const char*);
     virtual void read (const string&);
+
+    // operations on matrix
     template <class Tf> friend matrix<Tf> kronecker (int); // return the identity matrix (ie. i==j -> 1 and i!=j -> 0)
     matrix<T>& approximation ();
     virtual void permute_rows (int,int);     // permutes matrix rows
@@ -171,6 +174,7 @@ template <class T> class matrix
     template <class Tf> friend matrix<Tf> submatrix (const matrix<Tf>&, int, int, int, int);
     template <class Tf> friend matrix<Tf> merge_rows (const matrix<Tf>&, const matrix<Tf>&);
     template <class Tf> friend matrix<Tf> merge_columns (const matrix<Tf>&, const matrix<Tf>&);
+    T p_norm (int); // computes p-norm for matrix
 };
 
 
@@ -445,10 +449,12 @@ template <class Tf> vector<Tf> operator * (const matrix<Tf>& mat, const vector<T
 //------------------------------------------------------------------------------------
 { 
   assert (mat.nbColumns == v.dim()); // checks validity of the operation
+
   vector<Tf> u(mat.nbRows); // allocates memory and set vector components to zero
   for (int i=1; i<=mat.nbRows; i++)
       for (int j=1; j<=mat.nbColumns;j++)
            u[i] += mat(i,j) * v[j];
+
   return u;
 }
 
@@ -458,10 +464,12 @@ template <class Tf> vector<Tf> operator * (const vector<Tf>& v, const matrix<Tf>
 //------------------------------------------------------------------------------------
 { 
   assert (mat.nbRows == v.dim());
+
   vector<Tf> u(mat.nbColumns); // allocates memory and set vector components to zero
   for (int i=1; i<=mat.nbColumns; i++)
       for (int j=1; j<=mat.nbRows; j++)
           u[i] += v[j] * mat(j,i);
+
   return u;
 }
 
@@ -472,6 +480,7 @@ template <class Tf> matrix<Tf> operator / (const matrix<Tf>& mat1, const matrix<
 {
   matrix<Tf> mt = mat1;
   mt *= gauss(mat2);
+
   return mt;
 }
 
@@ -482,6 +491,7 @@ template <class Tf> matrix<Tf> operator / (const Tf& elt, const matrix<Tf>& mat)
 {
   matrix<Tf> mt = gauss(mat);
   mt*= elt;
+
   return mt;
 }
 
@@ -491,8 +501,10 @@ template <class Tf> matrix<Tf> operator / (const matrix<Tf>& mat, const Tf& elt)
 //------------------------------------------------------------------------------
 {
   assert (elt!=0); // can't divide by zero !
+
   matrix<Tf> mp = mat;
   mp /= elt;
+
   return mp;
 }
 
@@ -518,46 +530,55 @@ template <class Tf> matrix<Tf> operator ^ (const vector<Tf>& v, const vector<Tf>
 //----------------------------------------------------------------------------------
 {
   assert (v.dim() && w.dim());
+
   matrix<Tf> mat(v.dim(),w.dim(),false); // allocates memory for columns (no initialization)
   for (int i=1; i<=v.dim(); i++)
     for (int j=1; j<=w.dim(); j++)
       mat (i,j) = v[i]*w[j];
+
   return mat;
 }
 
 
-//-------------------------------------------------------------------
-template <class T> void matrix<T>::operator += (const matrix<T>& mat)
-//-------------------------------------------------------------------
+//-------------------------------------------------------------------------
+template <class T> matrix<T>& matrix<T>::operator += (const matrix<T>& mat)
+//-------------------------------------------------------------------------
 {
   assert(nbRows && nbColumns);
   assert ((nbRows==mat.nbRows) && (nbColumns==mat.nbColumns)); // matrices must have the same size
+
   // the following command allows the following operation : matrix += symatrix, so use it instead of mrows[i] += mat[i+1];
   for (int i=1; i<=nbRows; i++)
     for (int j=1; j<=nbColumns; j++)
       (*this)(i,j) += mat(i,j);
+
+  return *this;
 }
 
 
-//-------------------------------------------------------------------
-template <class T> void matrix<T>::operator -= (const matrix<T>& mat)
-//-------------------------------------------------------------------
+//-------------------------------------------------------------------------
+template <class T> matrix<T>& matrix<T>::operator -= (const matrix<T>& mat)
+//-------------------------------------------------------------------------
 {
   assert(nbRows && nbColumns);
   assert ((nbRows==mat.nbRows) && (nbColumns==mat.nbColumns)); // matrices must have the same size
+
   // the following command allows the following operation : matrix += symatrix, so use it instead of mrows[i] += mat[i+1];
   for (int i=1; i<=nbRows; i++)
     for (int j=1; j<=nbColumns; j++)
       (*this)(i,j) -= mat(i,j);
+
+  return *this;
 }
 
 
-//-------------------------------------------------------------------
-template <class T> void matrix<T>::operator *= (const matrix<T>& mat)
-//-------------------------------------------------------------------
+//-------------------------------------------------------------------------
+template <class T> matrix<T>& matrix<T>::operator *= (const matrix<T>& mat)
+//-------------------------------------------------------------------------
 {
   assert ((nbColumns==mat.nbRows)&&(nbColumns==mat.nbColumns));
   assert (nbColumns==(*this)[1].dim()); // this operator is not defined for symmatrix on the left side of the equation
+
   matrix<T> mt(nbRows,nbColumns); // allocates memory and set matrix components to zero
 
   for (int i=1; i<=nbRows; i++)
@@ -568,38 +589,45 @@ template <class T> void matrix<T>::operator *= (const matrix<T>& mat)
   delete [] mrows;
   mrows = mt.mrows;
   mt.nbRows = mt.nbColumns = 0;
+
+  return *this;
 }
 
 
-//-----------------------------------------------------------
-template <class T> void matrix<T>::operator *= (const T& elt)
-//-----------------------------------------------------------
+//-----------------------------------------------------------------
+template <class T> matrix<T>& matrix<T>::operator *= (const T& elt)
+//-----------------------------------------------------------------
 {
   assert (nbRows && nbColumns);
+
   for (int i=1; i<=nbRows; i++)
       (*this)[i] *= elt;
+
+  return *this;
 }
 
 
-//-------------------------------------------------------------------
-template <class T> void matrix<T>::operator /= (const matrix<T>& mat)
-//-------------------------------------------------------------------
+//-------------------------------------------------------------------------
+template <class T> matrix<T>& matrix<T>::operator /= (const matrix<T>& mat)
+//-------------------------------------------------------------------------
 {
   // a modifier et optimiser - cf operateur /
   assert (nbColumns==(*this)[1].dim()); // this operator is not defined for symmatrix on the left side of the equation
 
-  *this *= gauss(mat);
+  return   *this *= gauss(mat);
 }
 
 
-//-----------------------------------------------------------
-template <class T> void matrix<T>::operator /= (const T& elt)
-//-----------------------------------------------------------
+//-----------------------------------------------------------------
+template <class T> matrix<T>& matrix<T>::operator /= (const T& elt)
+//-----------------------------------------------------------------
 {
   assert (elt);
 
   for (int i=1; i<=nbRows; i++)
     (*this)[i] /= elt;
+
+  return *this;
 }
 
 
@@ -788,9 +816,9 @@ template <class Tf> matrix<Tf> gauss (matrix<Tf> mat)
 }
 
 
-//--------------------------------------------
-template <class Tf> bool inv (matrix<Tf>& mat)
-//--------------------------------------------
+//-----------------------------------------------------------------------
+template <class Tf> bool inv (matrix<Tf>& mat, long double eps = epsilon)
+//-----------------------------------------------------------------------
 {
   assert (mat.nbRows == mat.nbColumns); // matrix must be squared
   matrix <Tf> matinv =& Id2<Tf>(mat.nbRows); // initializes matinv as the identity matrix
@@ -807,7 +835,7 @@ template <class Tf> bool inv (matrix<Tf>& mat)
 	    posx = i;
 	  }
 
-      if (abs(pivot) < epsilon)
+      if (abs(pivot) <= eps)
 	{ mat = mat_copy;
 	  return false;
 	}
@@ -845,6 +873,9 @@ template <class Tf> bool inv (matrix<Tf>& mat)
     }
 
   mat =& matinv;
+
+  //  if (eps != epsilon) assert (mat * mat_copy == Id2<Tf>(mat.Rows()));
+
   return true;
 }
 
@@ -903,6 +934,33 @@ template <class T> matrix<T> gauss_pt (matrix<T> mat)
 */
 
 
+/*!
+  \brief Computes the p-norm for matrix.
+
+  Let \f$ A \f$ be a \f$ n \times m \f$ matrix. Then, its p-norm is given by: \n
+  \f$ \left\| A \right\|_p = \displaystyle \left( \sum_{i=1}^{n} \sum_{j=1}^{m} \left| a_{ij} \right|^p \right)^{1/p} \f$.\n
+  The special case p = 2 is the Frobenius norm also called the Hilbert-Schmidt norm.
+
+  \param p value for p-norm
+  \return \f$ \left\| A \right\|_p \f$
+
+*/
+
+//--------------------------------------------
+template <class T> T matrix<T>::p_norm (int p)
+//--------------------------------------------
+{
+  assert (p > 0);
+
+  T sum = 0;
+  for (int i=1; i<=nbRows; i++)
+    for (int j=1; j<=nbColumns; j++)
+      sum += pow (mrows[i-1][j], p);
+
+  return pow (sum, (long double) 1/p);
+}
+
+
 /*!
   \brief Writes matrix to hard disk.
 
