@@ -2,7 +2,7 @@
  *                 Mathematical Object Library                 *
  *       class spmatrix : declarations for sparse matrix       *
  *                    simula+@metz.ensam.fr                    *
- *		     GNU/linux version 2.2.0                   *
+ *		     GNU/linux version 2.4.0                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright © 2004,2005 LAGOUCHE Sophie
@@ -111,9 +111,9 @@
            copyright \htmlonly &#169; \endhtmlonly 2006, 2007, 2008, 2009 Christophe COLLARD \n
 	   copyright \htmlonly &#169; 2004, 2005, 2006, 2007, 2008, 2009 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
 	   copyright \htmlonly &#169; 2004, 2005, 2006, 2007 Laboratoire de Math&#233;matiques et ses Applications de Valenciennes (LAMAV) \endhtmlonly
-  \version 2.2.0
-  \date 2004-2009
-  \bug none
+  \version 2.4.0
+  \date 2004-2010
+  \bug multiple allocation does not work with g++ compiler (ex. A(2,4) = A(3,3) = A(5,5) = 2;)
   \warning none
 */
 
@@ -213,10 +213,10 @@ template <class T> class spmatrix : public matrix<T>
     template <class Tf> friend matrix<Tf>   operator * (const spmatrix<Tf>&, const spmatrix<Tf>&); // overloads sparse matrix * sparse matrix
     template <class Tf> friend spmatrix<Tf> operator / (const spmatrix<Tf>&, const Tf&); // overloads sparse matrix / scalar
     template <class Tf> friend Tf operator | (const spmatrix<Tf>&, const spmatrix<Tf>&); // overloads sparse matrix | sparse matrix
-    void operator += (const spmatrix<T>&); // overloads += operator for matrix
-    void operator -= (const spmatrix<T>&); // overloads -= operator for matrix
-    void operator *= (const T&); // overloads *= operator between matrix and scalar
-    void operator /= (const T&); // overloads /= operator between matrix and scalar
+    spmatrix<T>& operator += (const spmatrix<T>&); // overloads += operator for matrix
+    spmatrix<T>& operator -= (const spmatrix<T>&); // overloads -= operator for matrix
+    spmatrix<T>& operator *= (const T&);           // overloads *= operator between matrix and scalar
+    spmatrix<T>& operator /= (const T&);           // overloads /= operator between matrix and scalar
     template <class Tf> friend bool operator == (const spmatrix<Tf>&, const spmatrix<Tf>&); // compares 2 sparse matrices
     template <class Tf> friend bool operator != (const spmatrix<Tf>&, const spmatrix<Tf>&);
 
@@ -1232,13 +1232,13 @@ template <class Tf> Tf operator | (const spmatrix<Tf>& mat1, const spmatrix<Tf>&
   \param mat sparse matrix to add
 */
 
-//-----------------------------------------------------------------------
-template <class T> void spmatrix<T>::operator += (const spmatrix<T>& mat)
-//-----------------------------------------------------------------------
+//-------------------------------------------------------------------------------
+template <class T> spmatrix<T>& spmatrix<T>::operator += (const spmatrix<T>& mat)
+//-------------------------------------------------------------------------------
 {
   assert ( (nbRows == mat.nbRows)  &&  (nbColumns == mat.nbColumns) );
 
-  (*this) &= (*this) + mat;	//operator&= is used because the number of non-zero elements per row may change
+  return (*this) &= (*this) + mat;	//operator&= is used because the number of non-zero elements per row may change
 }
 
 
@@ -1248,13 +1248,13 @@ template <class T> void spmatrix<T>::operator += (const spmatrix<T>& mat)
   \param mat sparse matrix to substract
 */
 
-//-----------------------------------------------------------------------
-template <class T> void spmatrix<T>::operator -= (const spmatrix<T>& mat)
-//-----------------------------------------------------------------------
+//-------------------------------------------------------------------------------
+template <class T> spmatrix<T>& spmatrix<T>::operator -= (const spmatrix<T>& mat)
+//-------------------------------------------------------------------------------
 {
   assert ((nbRows == mat.nbRows) && (nbColumns == mat.nbColumns));
 
-  (*this) &= (*this) - mat;	//operator&= is used because the size of non-zero elements per row may change
+  return (*this) &= (*this) - mat;	//operator&= is used because the size of non-zero elements per row may change
 }
 
 
@@ -1264,9 +1264,9 @@ template <class T> void spmatrix<T>::operator -= (const spmatrix<T>& mat)
  \param elt scalar
 */
 
-//-------------------------------------------------------------
-template <class T> void spmatrix<T>::operator *= (const T& elt)
-//-------------------------------------------------------------
+//---------------------------------------------------------------------
+template <class T> spmatrix<T>& spmatrix<T>::operator *= (const T& elt)
+//---------------------------------------------------------------------
 {
   assert (nbRows && rows_max_elements);
 
@@ -1282,6 +1282,8 @@ template <class T> void spmatrix<T>::operator *= (const T& elt)
       for (int i=1; i<=nbRows; i++)
 	mrows[i-1] *= elt;
     }
+
+  return *this;
 }
 
 
@@ -1291,9 +1293,9 @@ template <class T> void spmatrix<T>::operator *= (const T& elt)
   \param elt scalar
 */
 
-//-------------------------------------------------------------
-template <class T> void spmatrix<T>::operator /= (const T& elt)
-//-------------------------------------------------------------
+//---------------------------------------------------------------------
+template <class T> spmatrix<T>& spmatrix<T>::operator /= (const T& elt)
+//---------------------------------------------------------------------
 {
   assert (nbRows && rows_max_elements);
   assert(abs(elt) > epsilon);
@@ -1302,6 +1304,8 @@ template <class T> void spmatrix<T>::operator /= (const T& elt)
 
   for (int i=1; i<=nbRows; i++)
     mrows[i-1] /= elt;
+
+  return *this;
 }
 
 
