@@ -2,7 +2,7 @@
  *                 Mathematical Object Library                 *
  *     class symmatrix:  declarations for symmetric matrix     *
  *                    simula+@metz.ensam.fr                    *
- *                   GNU/linux version 2.2.0                   *
+ *                   GNU/linux version 2.4.0                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright © 2002,2003,2004,2005,2006,2007,2008,2009 COLLARD Christophe
@@ -46,8 +46,8 @@
              copyright \htmlonly &#169; \endhtmlonly 2002, 2003, 2004 Emmanuel CREUSE \n
 	     copyright \htmlonly &#169; 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
 	     copyright \htmlonly &#169; 2002, 2003, 2004, 2005, 2006, 2007 Laboratoire de Math&#233;matiques et ses Applications de Valenciennes (LAMAV) \endhtmlonly
-    \version 2.2.0
-    \date 2002-2009
+    \version 2.4.0
+    \date 2002-2010
     \bug none
     \warning none
 */
@@ -122,10 +122,10 @@ template <class T> class symmatrix : public virtual matrix<T>
     //    S / S' is not symmetric when S and S' are symmetric
     template <class Tf> friend symmatrix<Tf> operator / (const symmatrix<Tf>&, const Tf&);  // overloads symmatrix / scalar
     template <class Tf> friend symmatrix<Tf> operator / (const Tf&, const symmatrix<Tf>&);  // overloads scalar / symmatrix
-    void operator += (const symmatrix<T>&); // overloads += operator for symmatrix/symmatrix
-    void operator += (const matrix<T>&);    // overloads += operator for symmatrix/matrix (which must be symmetric)
-    void operator -= (const symmatrix<T>&); // overloads -= operator for symmatrix/symmatrix
-    void operator -= (const matrix<T>&);    // overloads -= operator for symmatrix/matrix (which must be symmetric)
+    symmatrix<T>& operator += (const symmatrix<T>&); // overloads += operator for symmatrix/symmatrix
+    symmatrix<T>& operator += (const matrix<T>&);    // overloads += operator for symmatrix/matrix (which must be symmetric)
+    symmatrix<T>& operator -= (const symmatrix<T>&); // overloads -= operator for symmatrix/symmatrix
+    symmatrix<T>& operator -= (const matrix<T>&);    // overloads -= operator for symmatrix/matrix (which must be symmetric)
     template <class Tf> friend bool operator == (const symmatrix<Tf>&, const symmatrix<Tf>&); // optimizes == operator of symmatrix
     template <class Tf> friend bool operator != (const symmatrix<Tf>&, const symmatrix<Tf>&); // same optimization
     friend symmatrix<T> t (const symmatrix<T>& mat) {return mat;}        // optimizes tranposition operator for symmatrix
@@ -348,9 +348,10 @@ template <class Tf> symmatrix<Tf> operator + (const symmatrix<Tf>& mat1, const s
 {
   assert ( mat1.Rows()==mat2.Rows() ); // matrices must have the same size
   assert (mat1.Rows() && mat1.Columns());
+
   symmatrix<Tf> mat = mat1;
-  mat += mat2;
-  return mat;
+
+  return mat += mat2;
 }
 
 
@@ -360,9 +361,10 @@ template <class Tf> symmatrix<Tf> operator - (const symmatrix<Tf>& mat1, const s
 {
   assert ( mat1.Rows()==mat2.Rows() ); // matrices must have the same size
   assert (mat1.Rows() && mat1.Columns());
+
   symmatrix<Tf> mat = mat1;
-  mat -= mat2;
-  return mat;
+
+  return mat -= mat2;
 }
 
 
@@ -371,9 +373,10 @@ template <class Tf> symmatrix<Tf> operator * (const symmatrix<Tf>& mat, const Tf
 //------------------------------------------------------------------------------------
 {
   assert (mat.nbRows && mat.nbColumns);
+
   symmatrix<Tf> mp = mat;
-  mp *= elt;
-  return mp;
+
+  return mp *= elt;
 }
 
 
@@ -382,9 +385,10 @@ template <class Tf> symmatrix<Tf> operator * (const Tf& elt, const symmatrix<Tf>
 //------------------------------------------------------------------------------------
 {
   assert (mat.nbRows && mat.nbColumns);
+
   symmatrix<Tf> mp = mat;
-  mp *= elt;
-  return mp;
+
+  return mp *= elt;
 }
 
 
@@ -394,9 +398,10 @@ template <class Tf> symmatrix<Tf> operator / (const symmatrix<Tf>& mat, const Tf
 {
   assert (mat.nbRows&&mat.nbColumns);
   assert (elt); // can't divide by zero
+
   symmatrix<Tf> mp = mat;
-  mp /= elt;
-  return mp;
+
+  return mp /= elt;
 }
 
 
@@ -406,59 +411,72 @@ template <class Tf> symmatrix<Tf> operator / (const Tf& elt, const symmatrix<Tf>
 {
   assert (mat.nbRows&&mat.nbColumns);
   assert (elt); // can't divide by zero
+
   symmatrix<Tf> mp =& gauss(mat);
-  mp *= elt;
-  return mp;
+
+  return mp *= elt;
 }
 
 
-//-------------------------------------------------------------------------
-template <class T> void symmatrix<T>::operator += (const symmatrix<T>& mat)
-//-------------------------------------------------------------------------
+//----------------------------------------------------------------------------------
+template <class T> symmatrix<T>& symmatrix<T>::operator += (const symmatrix<T>& mat)
+//----------------------------------------------------------------------------------
 {
   assert(nbRows && nbColumns);
   assert ((nbRows==mat.nbRows) && (nbColumns==mat.nbColumns)); // matrices must have the same size
+
   for (int i=0; i<nbRows; i++)
     mrows[i] += mat.mrows[i];
+
+  return *this;
 }
 
 
-//----------------------------------------------------------------------
-template <class T> void symmatrix<T>::operator += (const matrix<T>& mat)
-//----------------------------------------------------------------------
+//-------------------------------------------------------------------------------
+template <class T> symmatrix<T>& symmatrix<T>::operator += (const matrix<T>& mat)
+//-------------------------------------------------------------------------------
 {
   assert(nbRows && nbColumns);
   assert ( (nbRows==mat.Rows()) && (nbColumns==mat.Columns()) ); // matrices must have the same size
+
   for (int i=1; i<=nbRows; i++)
     for (int j=1; j<=i; j++)
       { assert(abs(mat(i,j)-mat(j,i))<epsilon);
         (*this)(i,j) += mat(i,j);
       }
+
+  return *this;
 }
 
 
-//-------------------------------------------------------------------------
-template <class T> void symmatrix<T>::operator -= (const symmatrix<T>& mat)
-//-------------------------------------------------------------------------
+//----------------------------------------------------------------------------------
+template <class T> symmatrix<T>& symmatrix<T>::operator -= (const symmatrix<T>& mat)
+//----------------------------------------------------------------------------------
 {
   assert(nbRows && nbColumns);
   assert ((nbRows==mat.nbRows) && (nbColumns==mat.nbColumns)); // matrices must have the same size
+
   for (int i=0; i<nbRows; i++)
     mrows[i] -= mat.mrows[i];
+
+  return *this;
 }
 
 
-//----------------------------------------------------------------------
-template <class T> void symmatrix<T>::operator -= (const matrix<T>& mat)
-//----------------------------------------------------------------------
+//-------------------------------------------------------------------------------
+template <class T> symmatrix<T>& symmatrix<T>::operator -= (const matrix<T>& mat)
+//-------------------------------------------------------------------------------
 {
   assert ((nbRows==mat.Rows()) && (nbColumns==mat.Columns())); // matrices must have the same size
   assert((nbRows)&&(nbColumns));
+
   for (int i=1; i<=nbRows; i++)
     for (int j=i; j<=nbColumns; j++)
       { assert(abs(mat(i,j)-mat(j,i))<epsilon);
         (*this)(i,j) -= mat(i,j);
       }
+
+  return *this;
 }
 
 
