@@ -1,13 +1,12 @@
-
 /***************************************************************
  *                 Mathematical Object Library                 *
  *class symtensor2:declarations for 2nd order symmetric tensors*
  *                    simula+@metz.ensam.fr                    *
- *		     GNU/linux version 2.2.0                   *
+ *		     GNU/linux version 2.4.0                   *
  *            software under General Public License            *
  ***************************************************************
- * copyright © 2006,2007,2008,2009 COLLARD Christophe
- * copyright © 2006,2007,2008,2009 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
+ * copyright © 2006,2007,2008,2009,2010 COLLARD Christophe
+ * copyright © 2006,2007,2008,2009,2010 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
  * copyright © 2006,2007 Laboratoire de Mathématiques et ses Applications de Valenciennes (LAMAV)
  ***************************************************************/
 
@@ -43,11 +42,11 @@
 
     Let \f$ E \f$ be a Euclidean space of dimension \f$ n \f$ with basis vectors \f$ \{ e_i \} = (e_1,\ldots,e_n) \f$. A \f$ 2^\text{nd} \f$ order Euclidean tensor \f$ T \f$ is defined as a bilinear application from \f$ E \times E \longrightarrow \mathbb{R} \f$. We use a matrix storage for these tensors. All the basic operations are then already defined, so we just add some specific tensors operators to this class. \n
 
-    \author copyright \htmlonly &#169; \endhtmlonly 2006, 2007, 2008, 2009 Christophe COLLARD \n
-	    copyright \htmlonly &#169; 2006, 2007, 2008, 2009 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
+    \author copyright \htmlonly &#169; \endhtmlonly 2006, 2007, 2008, 2009, 2010 Christophe COLLARD \n
+	    copyright \htmlonly &#169; 2006, 2007, 2008, 2009, 2010 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
 	    copyright \htmlonly &#169; 2006, 2007 Laboratoire de Math&#233;matiques et ses Applications de Valenciennes (LAMAV) \endhtmlonly
-    \version 2.2.0
-    \date 2006-2009
+    \version 2.4.0
+    \date 2006-2010
     \bug none
     \warning none
 */
@@ -78,21 +77,17 @@ namespace mol
 {
 
 
-//==========================================================================
-template <class T> class symtensor2 : public symmatrix<T>, public tensor2<T>
-//==========================================================================
+//==================================================================================
+template <class T> class symtensor2 : public symmatrix<T>, public virtual tensor2<T>
+//==================================================================================
 {
-  using matrix<T>::nbRows;
-  using matrix<T>::nbColumns;
-  using matrix<T>::mrows;
-
-  public :  
+  public:
     symtensor2 () : symmatrix<T> () {}  // default constructor
-    symtensor2 (int i, bool init=true, T value=0) : symmatrix<T> (i,i,init,value) {}         // this constructor allocates memory
-    symtensor2 (int i, int j, bool init=true, T value=0) : symmatrix<T> (i,j,init,value) {}  // this constructor allocates memory
-    symtensor2 (const matrix<T>& mat) : symmatrix<T> (mat) {}       // translation constructor (converts matrix    -> symtensor2)
-    symtensor2 (const symmatrix<T>& smat) : symmatrix<T> (smat) {}  // translation constructor (converts symmatrix -> symtensor2)
-    symtensor2 (const tensor2<T>& tsr);                             // translation constructor (converts tensor2   -> symtensor2)
+    symtensor2 (int i, bool init=true, T value=0) : symmatrix<T> (i, i, init, value) {}         // this constructor allocates memory
+    symtensor2 (int i, int j, bool init=true, T value=0) : symmatrix<T> (i, j, init, value) {}  // this constructor allocates memory
+    symtensor2 (const matrix<T>& mat) : symmatrix<T> (mat) {}       // cast constructor (converts matrix    -> symtensor2)
+    symtensor2 (const symmatrix<T>& smat) : symmatrix<T> (smat) {}  // cast constructor (converts symmatrix -> symtensor2)
+    symtensor2 (const tensor2<T>& tsr);                             // cast constructor (converts tensor2   -> symtensor2)
     symtensor2 (symtensor2<T>* st) : symmatrix<T>(st) {}            // copy constructor for temporary objects
     symtensor2 (symmatrix<T>* smat)  : symmatrix<T>(smat) {}        // copy constructor for temporary objects
     ~symtensor2 (){}        // destructor
@@ -112,21 +107,14 @@ template <class T> class symtensor2 : public symmatrix<T>, public tensor2<T>
 */
 
 //------------------------------------------------------------------
-template <class T> symtensor2<T>::symtensor2 (const tensor2<T>& tsr)
+template <class T> symtensor2<T>::symtensor2 (const tensor2<T>& tsr) : symmatrix<T> (tsr.Rows(), tsr.Columns(), false)
 //------------------------------------------------------------------
 {
-  assert (tsr.nbRows && tsr.nbColumns);
-  assert (tsr.nbRows==tsr.nbColumns);  // this tensor must be a squared one in order to be symmetric
-
-  mrows = new vector<T> [nbRows=tsr.nbRows];
-  nbColumns = tsr.nbColumns;
-  for (int i=1; i<=tsr.nbRows; i++)
-    { mrows[i-1] = i;
-      for (int j=1; j<=i; j++)
-	{ assert (abs(tsr(i,j)-tsr(j,i))<epsilon); // needed by symtensor2 = tensor2
-	  (*this)(i,j) = tsr(i,j);
-	}
-    }
+  for (int i=1; i<=tsr.Rows(); i++)
+    for (int j=1; j<=i; j++)
+      { assert (abs(tsr(i,j)-tsr(j,i))<epsilon); // needed by symtensor2 = tensor2
+	(*this)(i,j) = tsr(i,j);
+      }
 }
 
 
