@@ -2,7 +2,7 @@
  *                 Mathematical Object Library                 *
  *     class tensor2 : declarations for 2nd order tensors      *
  *                    simula+@metz.ensam.fr                    *
- *		     GNU/linux version 2.2.0                   *
+ *		     GNU/linux version 2.4.0                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright © 2004,2005,2006,2008,2009 COLLARD Christophe
@@ -45,8 +45,8 @@
     \author copyright \htmlonly &#169; \endhtmlonly 2004, 2005, 2006, 2008, 2009 Christophe COLLARD \n
 	    copyright \htmlonly &#169; 2004, 2005, 2006, 2008, 2009 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
 	    copyright \htmlonly &#169; 2004, 2005, 2006 Laboratoire de Math&#233;matiques et ses Applications de Valenciennes (LAMAV) \endhtmlonly
-    \version 2.2.0
-    \date 2004-2009
+    \version 2.4.0
+    \date 2004-2010
     \bug none
     \warning none
 */
@@ -85,7 +85,8 @@ template <class T> class tensor2 : public virtual matrix<T>
     tensor2 (int i, bool init=true, T value=0) : matrix<T> (i,i,init,value) {}         // this constructor allocates memory
     tensor2 (int i, int j, bool init=true, T value=0) : matrix<T> (i,j,init,value) {}  // this constructor allocates memory
     tensor2 (const tensor2<T>& t) : matrix<T>(t) {}  // copy constructor
-    tensor2 (const matrix<T>&);                      // translation constructor (converts matrix -> tensor)
+    tensor2 (const matrix<T>&);                      // cast constructor (converts matrix -> tensor)
+    //    tensor2 (const symtensor2<T>&) {}
     tensor2 (tensor2<T>* t) : matrix<T>(t) {}        // copy constructor for temporary objects
     tensor2 (matrix<T>* m)  : matrix<T>(m) {}        // copy constructor for temporary objects
     ~tensor2 (){}  // destructor
