@@ -2,7 +2,7 @@
  *                 Mathematical Object Library                 *
  *      class tensor3 : declarations for 3rd order tensors     *
  *                    simula+@metz.ensam.fr                    *
- *                   GNU/linux version 2.2.0                   *
+ *                   GNU/linux version 2.4.0                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright © 2004,2005,2006,2007,2008,2009 COLLARD Christophe
@@ -43,8 +43,8 @@
     \author copyright \htmlonly &#169; \endhtmlonly 2004, 2005, 2006, 2007, 2008, 2009 Christophe COLLARD \n
 	    copyright \htmlonly &#169; 2004, 2005, 2006, 2007, 2008, 2009 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
 	    copyright \htmlonly &#169; 2004, 2005, 2006, 2007 Laboratoire de Math&#233;matiques et ses Applications de Valenciennes (LAMAV) \endhtmlonly
-    \version 2.2.0
-    \date 2004-2009
+    \version 2.4.0
+    \date 2004-2010
     \bug none
     \warning none
 */
@@ -138,6 +138,11 @@ template <class T> class tensor3
 //=====Public methods for tensor3==============================================
 
 
+/*!
+  \brief Default constructor
+
+*/
+
 //---------------------------------------
 template <class T> tensor3<T>::tensor3 ()
 //---------------------------------------
@@ -146,6 +151,14 @@ template <class T> tensor3<T>::tensor3 ()
 }
 
 
+/*!
+  \brief Constructor
+
+  \param sz space dimensions (\f$\Bbb{R}^{sz} \times \Bbb{R}^{sz} \times \Bbb{R}^{sz} \f$) for the \f$ 3^\text{rd} \f$ order tensor
+  \param init boolean that indicates if the constructor has to initialize the tensor coordinates (default value is true)
+  \param value value for tensor coordinates (default value is 0)
+*/
+
 //-----------------------------------------------------------------
 template <class T> tensor3<T>::tensor3 (int sz, bool init, T value)
 //-----------------------------------------------------------------
@@ -248,6 +261,18 @@ template <class T> tensor2<T>& tensor3<T>::operator [] (int k) const
 }
 
 
+/*!
+  \brief Standard operator = for \f$ 3^\text{rd} \f$ order tensor
+
+  If both tensors are fully defined (size + table of coordinates), it checks that the space dimension is the same for the first coordinate and then uses the operator = of \f$ 2^\text{nd} \f$ order tensors. If the left hand side tensor is partially defined (space dimension of the first coordinate of the tensor is zero) then it allocates memory for this tensor and copies the coordinates using operator = of \f$ 2^\text{nd} \f$ order tensors. \n
+
+  \param tn \f$ 3^\text{rd} \f$ order tensor to copy
+
+  \return reference of the left hand side tensor (for multiple equalities)
+
+  \sa mol::tensor2::operator=(const tensor2&)
+*/
+
 //--------------------------------------------------------------------------
 template <class T> tensor3<T>& tensor3<T>::operator = (const tensor3<T>& tn)
 //--------------------------------------------------------------------------
