@@ -2,11 +2,11 @@
  *                 Mathematical Object Library                 *
  *      class tensor4 : declarations for 4th order tensors     *
  *                    simula+@metz.ensam.fr                    *
- *                   GNU/linux version 2.2.0                   *
+ *                   GNU/linux version 2.4.0                   *
  *            software under General Public License            *
  ***************************************************************
- * copyright © 2004,2005,2006,2007,2008,2009 COLLARD Christophe
- * copyright © 2004,2005,2006,2007,2008,2009 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
+ * copyright © 2004,2005,2006,2007,2008,2009,2010 COLLARD Christophe
+ * copyright © 2004,2005,2006,2007,2008,2009,2010 Laboratoire de Physique et Mécanique des Matériaux (LPMM - CNRS)
  * copyright © 2004,2005,2006,2007 Laboratoire de Mathématiques et ses Applications de Valenciennes (LAMAV)
  ***************************************************************/
 
@@ -40,11 +40,11 @@
     </FONT>
     \endhtmlonly
 
-    \author copyright \htmlonly &#169; \endhtmlonly 2004, 2005, 2006, 2007, 2008, 2009 Christophe COLLARD \n
-	    copyright \htmlonly &#169; 2004, 2005, 2006, 2007, 2008, 2009 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
+    \author copyright \htmlonly &#169; \endhtmlonly 2004, 2005, 2006, 2007, 2008, 2009, 2010 Christophe COLLARD \n
+	    copyright \htmlonly &#169; 2004, 2005, 2006, 2007, 2008, 2009, 2010 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
 	    copyright \htmlonly &#169; 2004, 2005, 2006, 2007 Laboratoire de Math&#233;matiques et ses Applications de Valenciennes (LAMAV) \endhtmlonly
-    \version 2.2.0
-    \date 2004-2009
+    \version 2.4.0
+    \date 2004-2010
     \bug none
     \warning none
 */
@@ -118,21 +118,33 @@ template <class T> class tensor4
     tensor4<T>& operator -= (const tensor4<T>&);     // overloads -= operator for tensors
     template <class Tf> friend bool operator == (const tensor4<Tf>&, const tensor4<Tf>&);  // compares 2 tensors
     template <class Tf> friend bool operator != (const tensor4<Tf>&, const tensor4<Tf>&);
+
+    //------------------------
     // operations with scalars
+    //------------------------
     tensor4<T>& operator *= (const T&);
     template <class Tf> friend tensor4<Tf> operator * (const tensor4<Tf>&, const Tf&);
     template <class Tf> friend tensor4<Tf> operator * (const Tf&, const tensor4<Tf>&);
+
+    //-------------------------
     // operations withs vectors
+    //-------------------------
     template <class Tf> friend tensor3<Tf> operator * (const tensor4<Tf>&, const vector<Tf>&);   // P_ijk = T_ijkl * V_l
     template <class Tf> friend tensor3<Tf> operator * (const vector<Tf>&, const tensor4<Tf>&);   // P_jkl = T_ijkl * V_i
     template <class Tf> friend tensor3<Tf> operator | (const tensor4<Tf>&, const vector<Tf>&);   // P_ijl = T_ijkl * V_k
     template <class Tf> friend tensor3<Tf> operator | (const vector<Tf>&, const tensor4<Tf>&);   // P_ikl = T_ijkl * V_j
     template <class Tf> friend tensor3<Tf> tensor_sum (const tensor4<Tf>&, const vector<Tf>&, int); // P = T : v
+
+    //------------------------------------------
     // operation with tensor2 (2nd order tensor)
+    //------------------------------------------
     template <class Tf> friend tensor2<Tf> operator || (const tensor4<Tf>&, const tensor2<Tf>&);  // P_ij = T_ijkl * M_kl
     template <class Tf> friend tensor2<Tf> operator || (const tensor2<Tf>&, const tensor4<Tf>&);  // P_kl = M_ij * T_ijkl
     template <class Tf> friend tensor4<Tf> operator ^  (const tensor2<Tf>&, const tensor2<Tf>&);  // T_ijkl = M1_ij M2_kl
+
+    //--------------------------------
     // operation with 3rd order tensor
+    //--------------------------------
     template <class Tf> friend tensor4<Tf> operator ^  (const tensor3<Tf>&, const tensor3<Tf>&); // T_ijkl = Pn,ij * Rn,kl = P_ij[n] * R_kl[n]
     template <class Tf> friend tensor3<Tf> operator || (const tensor3<Tf>&, const tensor4<Tf>&);  // P_n,ij = R_n,kl T_klij
     template <class Tf> friend tensor3<Tf> operator || (const tensor4<Tf>&, const tensor3<Tf>&);  // P_n,ij = T_ijkl R_n,kl
@@ -156,6 +168,11 @@ template <class T> class tensor4
 //=====Public methods for tensor4==============================================
 
 
+/*!
+  \brief Default constructor
+
+*/
+
 //---------------------------------------
 template <class T> tensor4<T>::tensor4 ()
 //---------------------------------------
@@ -164,6 +181,14 @@ template <class T> tensor4<T>::tensor4 ()
 }
 
 
+/*!
+  \brief Constructor
+
+  \param sz space dimensions (\f$\Bbb{R}^{sz} \times \Bbb{R}^{sz} \times \Bbb{R}^{sz} \times \Bbb{R}^{sz}\f$) for the \f$ 4^\text{th} \f$ order tensor
+  \param init boolean that indicates if the constructor has to initialize the tensor coordinates (default value is true)
+  \param value value for tensor coordinates (default value is 0)
+*/
+
 //-----------------------------------------------------------------
 template <class T> tensor4<T>::tensor4 (int sz, bool init, T value)
 //-----------------------------------------------------------------
@@ -175,6 +200,17 @@ template <class T> tensor4<T>::tensor4 (int sz, bool init, T value)
 }
 
 
+/*!
+  \brief Constructor
+
+  \param ntensor3 space dimension for the first component of the \f$ 4^\text{th} \f$ order tensor
+  \param ntensor2 space dimension for the second component of the \f$ 4^\text{th} \f$ order tensor
+  \param nrows space dimension for the third component of the \f$ 4^\text{th} \f$ order tensor
+  \param ncolumns space dimension for the fourth component of the \f$ 4^\text{th} \f$ order tensor
+  \param init boolean that indicates if the constructor has to initialize the tensor coordinates (default value is true)
+  \param value value for tensor coordinates (default value is 0)
+*/
+
 //---------------------------------------------------------------------------------------------------------------
 template <class T> tensor4<T>::tensor4 (int ntensor3, int ntensor2, int nrows, int ncolumns , bool init, T value)
 //---------------------------------------------------------------------------------------------------------------
@@ -186,6 +222,12 @@ template <class T> tensor4<T>::tensor4 (int ntensor3, int ntensor2, int nrows, i
 }
 
 
+/*!
+  \brief Copy constructor
+
+  \param tn \f$ 4^\text{th} \f$ order tensor to copy
+*/
+
 //-----------------------------------------------------------
 template <class T> tensor4<T>::tensor4 (const tensor4<T>& tn)
 //-----------------------------------------------------------
@@ -197,6 +239,21 @@ template <class T> tensor4<T>::tensor4 (const tensor4<T>& tn)
 }
 
 
+/*!
+  \brief Destructive copy constructor
+
+  Allocates the components from the right hand side (temporary) object to the left hand side object. \n
+  Since there is no copy of the coordinates of the (temporary) object, this object is destructed by this constructor.\n
+  Moreover, if we define the following tensors:\n
+  \f$ \textbf{tensor4$<$long double$>$ t1 (3, true, 1), t2 (5, true, 2);} \f$ \n
+  the definition of the tensor \n
+  \f$ \textbf{ tensor4$<$long double$>$ t3 (\&(t1 + t2));} \f$ \n
+  is faster than \n
+  \f$ \textbf{tensor4$<$long double$>$ t3 (t1 + t2);} \f$.
+
+  \param tn address of the tensor to allocate
+*/
+
 //-----------------------------------------------------
 template <class T> tensor4<T>::tensor4 (tensor4<T>* tn)
 //-----------------------------------------------------
@@ -209,6 +266,10 @@ template <class T> tensor4<T>::tensor4 (tensor4<T>* tn)
 }
 
 
+/*!
+  \brief Destructor
+*/
+
 //----------------------------------------
 template <class T> tensor4<T>::~tensor4 ()
 //----------------------------------------
@@ -220,6 +281,15 @@ template <class T> tensor4<T>::~tensor4 ()
 }
 
 
+/*!
+  \brief Partial memory allocation
+
+  Allocates memory for the first component only. \n
+  (don't use this function except if you know what you're doing !!)
+
+  \param ntensor3 space dimension for the first component of the symmetric \f$ 4^\text{th} \f$ order tensor
+*/
+
 //-------------------------------------------------------
 template <class T> void tensor4<T>::create (int ntensor3)
 //-------------------------------------------------------
@@ -231,6 +301,16 @@ template <class T> void tensor4<T>::create (int ntensor3)
 }
 
 
+/*!
+  \brief Full memory allocation without initialization of the tensor coordinates
+
+  Allocates memory without initilizing the tensor components to zero.
+  \param ntensor3 space dimension for the first component of the \f$ 4^\text{th} \f$ order tensor
+  \param ntensor2 space dimension for the second component of the \f$ 4^\text{th} \f$ order tensor
+  \param nrows space dimension for the third component of the \f$ 4^\text{th} \f$ order tensor
+  \param ncolumns space dimension for the fourth component of the \f$ 4^\text{th} \f$ order tensor
+*/
+
 //----------------------------------------------------------------------------------------------
 template <class T> void tensor4<T>::assign (int ntensor3, int ntensor2, int nrows, int ncolumns)
 //----------------------------------------------------------------------------------------------
@@ -243,6 +323,18 @@ template <class T> void tensor4<T>::assign (int ntensor3, int ntensor2, int nrow
 }
 
 
+/*!
+  \brief Returns the reference of a coordinate for a \f$ 4^\text{th} \f$ order tensor
+
+  Warning : we use the mathematical notations so the tensor coordinates are from 1 to the space dimension.
+  \param i first coordinate of the \f$ 4^\text{th} \f$ order tensor
+  \param j second coordinate of the \f$ 4^\text{th} \f$ order tensor
+  \param k third coordinate of the \f$ 4^\text{th} \f$ order tensor
+  \param l fourth coordinate of the \f$ 4^\text{th} \f$ order tensor
+
+  \return coordinate \f$ T(i,j,k,l) \f$ of the \f$ 4^\text{th} \f$ order tensor \f$ \bold{T} \f$
+*/
+
 //------------------------------------------------------------------------------
 template <class T> T& tensor4<T>::operator () (int i, int j, int k, int l) const
 //------------------------------------------------------------------------------
@@ -254,6 +346,16 @@ template <class T> T& tensor4<T>::operator () (int i, int j, int k, int l) const
 }
 
 
+/*!
+  \brief Returns the reference of a set  of coordinates for a \f$ 4^\text{th} \f$ order tensor
+
+  Warning : we use the mathematical notations so the tensor coordinates are from 1 to the space dimension.
+
+  \param i first coordinate of the \f$ 4^\text{th} \f$ order tensor
+
+  \return the reference of the \f$ 3^\text{rd} \f$ order tensor \f$ T(j,k,l) \f$ from the \f$ 4^\text{th} \f$ order tensor \f$ T(i,j,k,l) \f$
+*/
+
 //------------------------------------------------------------------
 template <class T> tensor3<T>& tensor4<T>::operator [] (int i) const
 //------------------------------------------------------------------
@@ -263,6 +365,18 @@ template <class T> tensor3<T>& tensor4<T>::operator [] (int i) const
 }
 
 
+/*!
+  \brief Standard operator = for \f$ 4^\text{th} \f$ order tensor
+
+  If both tensors are fully defined (size + table of coordinates), it checks that the space dimension is the same for the first coordinate and then uses the operator = of \f$ 3^\text{rd} \f$ order tensors. If the left hand side tensor is partially defined (space dimension of the first coordinate of the tensor is zero) then it allocates memory for this tensor and copies the coordinates using operator = of \f$ 3^\text{rd} \f$ order tensors. \n
+
+  \param tn \f$ 4^\text{th} \f$ order tensor to copy
+
+  \return reference of the left hand side tensor (for multiple equalities)
+
+  \sa mol::tensor3::operator=(const tensor3&)
+*/
+
 //--------------------------------------------------------------------------
 template <class T> tensor4<T>& tensor4<T>::operator = (const tensor4<T>& tn)
 //--------------------------------------------------------------------------
@@ -278,6 +392,17 @@ template <class T> tensor4<T>& tensor4<T>::operator = (const tensor4<T>& tn)
 }
 
 
+/*!
+  \brief Destructive operator = for \f$ 4^\text{th} \f$ order tensor
+
+  Makes a fast copy of tensor \f$ tn \f$ data (size, address of the coordinates table) and deletes \f$ tn \f$ (turns its size to zero). Note that we do not make any duplication of the coordinates table here. \n
+  This operator is generally used with temporary objects.
+
+  \param tn \f$ 4^\text{th} \f$ order tensor address
+
+  \return reference of the left hand side tensor (for multiple equalities)
+*/
+
 //--------------------------------------------------------------------
 template <class T> tensor4<T>& tensor4<T>::operator = (tensor4<T>* tn)
 //--------------------------------------------------------------------
@@ -296,6 +421,18 @@ template <class T> tensor4<T>& tensor4<T>::operator = (tensor4<T>* tn)
 }
 
 
+/*!
+  \brief Operator = for \f$ 4^\text{th} \f$ order tensors with different size
+
+  If the size of the right hand side tensor is zero, then we delete the table of coordinates of the left hand side tensor. \n
+  If both tensors are fully defined (size + table of coordinates) and have the same size then we only copy the coordinates. \n
+  If both tensors are fully defined but don't have the same size, then we delete the table of coordinates of the left hand side tensor and we reallocate memory for this table with the size of the right hand side tensor, then we copy the coordinates. \n
+
+  \param tn \f$ 4^\text{th} \f$ order tensor to copy
+
+  \return reference of the left hand side tensor (for multiple equalities)
+*/
+
 //---------------------------------------------------------------------------
 template <class T> tensor4<T>& tensor4<T>::operator &= (const tensor4<T>& tn)
 //---------------------------------------------------------------------------
@@ -318,6 +455,17 @@ template <class T> tensor4<T>& tensor4<T>::operator &= (const tensor4<T>& tn)
 }
 
 
+/*!
+  \brief Destructive operator = for \f$ 4^\text{th} \f$ order tensors with different size
+
+  Makes a fast copy of the tensor \f$ tn \f$ data (size, address of the coordinates table) and deletes \f$ tn \f$ (turns its size to zero). Note that we do not make any duplication of the coordinates table here. \n
+  This operator is generally used with temporary objects.
+
+  \param tn \f$ 4^\text{th} \f$ order tensor address
+
+  \return reference of the left hand side tensor (for multiple equalities)
+*/
+
 //---------------------------------------------------------------------
 template <class T> tensor4<T>& tensor4<T>::operator &= (tensor4<T>* tn)
 //---------------------------------------------------------------------
@@ -339,6 +487,18 @@ template <class T> tensor4<T>& tensor4<T>::operator &= (tensor4<T>* tn)
 }
 
 
+/*!
+  \brief Addition for \f$ 4^\text{th} \f$ order tensors
+
+  Computes the addition of the tensors \f$ \boldsymbol{a} = (a_{ijkl}) \f$ and \f$ \boldsymbol{b} = (b_{ijkl}) \f$ : \n
+  \f$ c_{ijkl} = a_{ijkl} + b_{ijkl} \f$, where \f$ \boldsymbol{c} = (c_{ijkl}) \f$.
+
+  \param t1 tensor \f$ \boldsymbol{a} \f$
+  \param t2 tensor \f$ \boldsymbol{b} \f$
+
+  \return \f$ \boldsymbol{c} = \boldsymbol{a} + \boldsymbol{b} \f$
+*/
+
 //---------------------------------------------------------------------------------------
 template <class Tf> tensor4<Tf> operator + (const tensor4<Tf>& t1, const tensor4<Tf>& t2)
 //---------------------------------------------------------------------------------------
@@ -350,6 +510,18 @@ template <class Tf> tensor4<Tf> operator + (const tensor4<Tf>& t1, const tensor4
 }
 
 
+/*!
+  \brief Subtraction for \f$ 4^\text{th} \f$ order tensors
+
+  Computes the subtraction of the tensors \f$ \boldsymbol{a} = (a_{ijkl}) \f$ and \f$ \boldsymbol{b} = (b_{ijkl}) \f$ : \n
+  \f$ c_{ijkl} = a_{ijkl} - b_{ijkl} \f$, where \f$ \boldsymbol{c} = (c_{ijkl}) \f$.
+
+  \param t1 tensor \f$ \boldsymbol{a} \f$
+  \param t2 tensor \f$ \boldsymbol{b} \f$
+
+  \return \f$ \boldsymbol{c} = \boldsymbol{a} - \boldsymbol{b} \f$
+*/
+
 //---------------------------------------------------------------------------------------
 template <class Tf> tensor4<Tf> operator - (const tensor4<Tf>& t1, const tensor4<Tf>& t2)
 //---------------------------------------------------------------------------------------
@@ -361,6 +533,18 @@ template <class Tf> tensor4<Tf> operator - (const tensor4<Tf>& t1, const tensor4
 }
 
 
+/*!
+  \brief Contracted product for \f$ 4^\text{th} \f$ order tensors
+
+  Computes the contracted product of the tensors \f$ \boldsymbol{a} = (a_{ijkl}) \f$ and \f$ \boldsymbol{b} = (b_{ijkl}) \f$ : \n
+  \f$ c = a_{ijkl} b_{ijkl} \f$, where the result \f$ c \f$ is a scalar.
+
+  \param t1 \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{a} \f$
+  \param t2 \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{b} \f$
+
+  \return \f$ c = \boldsymbol{a} | \boldsymbol{b} \f$
+*/
+
 //------------------------------------------------------------------------------
 template <class Tf> Tf operator | (const tensor4<Tf>& t1, const tensor4<Tf>& t2)
 //------------------------------------------------------------------------------
@@ -376,6 +560,18 @@ template <class Tf> Tf operator | (const tensor4<Tf>& t1, const tensor4<Tf>& t2)
 }
 
 
+/*!
+  \brief Contracted product for \f$ 4^\text{th} \f$ order tensors
+
+  Computes the contracted product of the tensors \f$ \boldsymbol{a} = (a_{ijkl}) \f$ and \f$ \boldsymbol{b} = (b_{ijkl}) \f$ : \n
+  \f$ c_{ijkl} = a_{ijpq} b_{pqkl} \f$, where \f$ \boldsymbol{c} = (c_{ijkl}) \f$.
+
+  \param t1 \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{a} \f$
+  \param t2 \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{b} \f$
+
+  \return \f$ \boldsymbol{c} = \boldsymbol{a} :: \boldsymbol{b} \f$
+*/
+
 //----------------------------------------------------------------------------------------
 template <class Tf> tensor4<Tf> operator || (const tensor4<Tf>& t1, const tensor4<Tf>& t2)
 //----------------------------------------------------------------------------------------
@@ -396,30 +592,66 @@ template <class Tf> tensor4<Tf> operator || (const tensor4<Tf>& t1, const tensor
 }
 
 
+/*!
+  \brief Add a \f$ 4^\text{th} \f$ order tensor
+
+  Adds the \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{R} = (R_i) \f$ to the current \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{T} = (T_i) \f$ : \n
+  \f$ T_i += R_i \f$  (which is equivalent to \f$ T_i = T_i + R_i \f$ but faster).
+
+  \param tn \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{R} \f$
+
+  \return \f$  \boldsymbol{T} =  \boldsymbol{T} +  \boldsymbol{R} \f$
+*/
+
 //---------------------------------------------------------------------------
-template <class T> tensor4<T>& tensor4<T>::operator += (const tensor4<T>& ts)
+template <class T> tensor4<T>& tensor4<T>::operator += (const tensor4<T>& tn)
 //---------------------------------------------------------------------------
 {
-  assert (size);  // the tensor must be defined
-  assert (size == ts.size);  // tensors must have the same size
+  assert (size);  // the tensor tn must be defined
+  assert (size == tn.size);  // tensors must have the same size
+
   for (int i=0; i<size; i++)
-    tsr[i] += ts.tsr[i];
+    tsr[i] += tn.tsr[i];
+
   return *this;
 }
 
 
+/*!
+  \brief Subtract a \f$ 4^\text{th} \f$ order tensor
+
+  Subtracts the \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{R} = (R_i) \f$ to the current \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{T} = (T_i) \f$ : \n
+  \f$ T_i -= R_i \f$  (which is equivalent to \f$ T_i = T_i - R_i \f$ but faster).
+
+  \param tn \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{R} \f$
+
+  \return \f$  \boldsymbol{T} =  \boldsymbol{T} -  \boldsymbol{R} \f$
+*/
+
 //---------------------------------------------------------------------------
-template <class T> tensor4<T>& tensor4<T>::operator -= (const tensor4<T>& ts)
+template <class T> tensor4<T>& tensor4<T>::operator -= (const tensor4<T>& tn)
 //---------------------------------------------------------------------------
 {
-  assert (size);  // the tensor must be defined
-  assert (size == ts.size);  // tensors must have the same size
+  assert (size);  // the tensor tn must be defined
+  assert (size == tn.size);  // tensors must have the same size
+
   for (int i=0; i<size; i++)
-    tsr[i] -= ts.tsr[i];
+    tsr[i] -= tn.tsr[i];
+
   return *this;
 }
 
 
+/*!
+  \brief Compares two \f$ 4^\text{th} \f$ order tensors
+
+  First compare tensor spaces and then compare its coordinates.
+  \param t1 \f$ 4^\text{th} \f$ order tensor \f$ t_1 \f$
+  \param t2 \f$ 4^\text{th} \f$ order tensor \f$ t_2 \f$
+
+  \return TRUE if \f$ t_1 = t_2 \f$ and FALSE if \f$ t_1 \neq t_2 \f$
+*/
+
 //---------------------------------------------------------------------------------
 template <class Tf> bool operator == (const tensor4<Tf>& t1, const tensor4<Tf>& t2)
 //---------------------------------------------------------------------------------
@@ -432,6 +664,16 @@ template <class Tf> bool operator == (const tensor4<Tf>& t1, const tensor4<Tf>&
 }
 
 
+/*!
+  \brief Compares two  \f$ 4^\text{th} \f$ order tensors
+
+  First compare tensor spaces and then compare its coordinates.
+  \param t1 \f$ 4^\text{th} \f$ order tensor \f$ t_1 \f$
+  \param t2 \f$ 4^\text{th} \f$ order tensor \f$ t_2 \f$
+
+  \return TRUE if \f$ t_1 \neq t_2 \f$ and FALSE if \f$ t_1 = t_2 \f$
+*/
+
 //---------------------------------------------------------------------------------
 template <class Tf> bool operator != (const tensor4<Tf>& t1, const tensor4<Tf>& t2)
 //---------------------------------------------------------------------------------
@@ -440,6 +682,20 @@ template <class Tf> bool operator != (const tensor4<Tf>& t1, const tensor4<Tf>&
 }
 
 
+//=====Public methods for tensor4 and scalars==================================
+
+
+/*!
+  \brief Multiplication by a scalar
+
+  Multiply the \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{T} = (t_{ijkl}) \f$ by a scalar a:
+  \f$ t_{ijkl} *= a \f$  (which is equivalent to \f$ t_{ijkl} = t_{ijkl} * a \f$ but faster).
+
+  \param elt scalar \f$ a \f$
+
+  \return \f$  \boldsymbol{T} = a \boldsymbol{T} \f$
+*/
+
 //-------------------------------------------------------------------
 template <class T> tensor4<T>& tensor4<T>::operator *= (const T& elt)
 //-------------------------------------------------------------------
@@ -451,6 +707,18 @@ template <class T> tensor4<T>& tensor4<T>::operator *= (const T& elt)
 }
 
 
+/*!
+  \brief Multiplication for \f$ 4^\text{th} \f$ order tensor with scalar
+
+  Computes the multiplication of the tensor \f$ \boldsymbol{T} = (t_{ijkl}) \f$ by a scalar \f$ a \f$ : \n
+  \f$ \boldsymbol{T} * a = t_{ijkl} * a \f$.
+
+  \param ts  \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{T} \f$
+  \param elt scalar \f$ a \f$
+
+  \return \f$ \boldsymbol{T} * a \f$
+*/
+
 //-------------------------------------------------------------------------------
 template <class Tf> tensor4<Tf> operator * (const tensor4<Tf>& ts, const Tf& elt)
 //-------------------------------------------------------------------------------
@@ -461,6 +729,18 @@ template <class Tf> tensor4<Tf> operator * (const tensor4<Tf>& ts, const Tf& elt
 }
 
 
+/*!
+  \brief Multiplication for \f$ 4^\text{th} \f$ order tensor with scalar
+
+  Computes the multiplication of the tensor \f$ \boldsymbol{T} = (t_{ijkl}) \f$ by a scalar \f$ a \f$ : \n
+  \f$ a * \boldsymbol{T} = a * t_{ijkl} \f$.
+
+  \param elt scalar \f$ a \f$
+  \param ts  \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{T} \f$
+
+  \return \f$ a * \boldsymbol{T} \f$
+*/
+
 //-------------------------------------------------------------------------------
 template <class Tf> tensor4<Tf> operator * (const Tf& elt, const tensor4<Tf>& ts)
 //-------------------------------------------------------------------------------
@@ -471,6 +751,21 @@ template <class Tf> tensor4<Tf> operator * (const Tf& elt, const tensor4<Tf>& ts
 }
 
 
+//=====Public methods for tensor4 and vectors==================================
+
+
+/*!
+  \brief Multiplication for \f$ 4^\text{th} \f$ order tensor with vector
+
+  Computes the multiplication of the tensor \f$ \boldsymbol{T} = (t_{ijkl}) \f$ with the vector \f$ \boldsymbol{v} = v_i \f$ : \n
+  \f$ \boldsymbol{T} * \boldsymbol{v} = t_{ijkl} * v_l \f$.
+
+  \param ts  \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{T} \f$
+  \param v vector \f$ \boldsymbol{v} \f$
+
+  \return \f$ \boldsymbol{T} * \boldsymbol{v} \f$
+*/
+
 //-------------------------------------------------------------------------------------
 template <class Tf> tensor3<Tf> operator * (const tensor4<Tf>& ts, const vector<Tf>& v)
 //-------------------------------------------------------------------------------------
@@ -487,6 +782,18 @@ template <class Tf> tensor3<Tf> operator * (const tensor4<Tf>& ts, const vector<
 }
 
 
+/*!
+  \brief Multiplication for vector with \f$ 4^\text{th} \f$ order tensor
+
+  Computes the multiplication of the vector \f$ \boldsymbol{v} = v_i \f$ with the tensor \f$ \boldsymbol{T} = (t_{ijkl}) \f$ : \n
+  \f$ \boldsymbol{v} * \boldsymbol{T} = v_i * t_{ijkl} \f$.
+
+  \param v vector \f$ \boldsymbol{v} \f$
+  \param ts  \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{T} \f$
+
+  \return \f$ \boldsymbol{v} * \boldsymbol{T} \f$
+*/
+
 //-------------------------------------------------------------------------------------
 template <class Tf> tensor3<Tf> operator * (const vector<Tf>& v, const tensor4<Tf>& ts)
 //-------------------------------------------------------------------------------------
@@ -500,6 +807,18 @@ template <class Tf> tensor3<Tf> operator * (const vector<Tf>& v, const tensor4<T
 }
 
 
+/*!
+  \brief Multiplication for \f$ 4^\text{th} \f$ order tensor with vector
+
+  Computes the multiplication of the tensor \f$ \boldsymbol{T} = (t_{ijkl}) \f$ with the vector \f$ \boldsymbol{v} = v_i \f$ : \n
+  \f$ \boldsymbol{T} | \boldsymbol{v} = t_{ijkl} * v_k \f$.
+
+  \param ts  \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{T} \f$
+  \param v vector \f$ \boldsymbol{v} \f$
+
+  \return \f$ \boldsymbol{T} | \boldsymbol{v} \f$
+*/
+
 //-------------------------------------------------------------------------------------
 template <class Tf> tensor3<Tf> operator | (const tensor4<Tf>& ts, const vector<Tf>& v)
 //-------------------------------------------------------------------------------------
@@ -515,6 +834,18 @@ template <class Tf> tensor3<Tf> operator | (const tensor4<Tf>& ts, const vector<
 }
 
 
+/*!
+  \brief Multiplication for vector with \f$ 4^\text{th} \f$ order tensor
+
+  Computes the multiplication of the vector \f$ \boldsymbol{v} = v_i \f$ with the tensor \f$ \boldsymbol{T} = (t_{ijkl}) \f$ : \n
+  \f$ \boldsymbol{v} | \boldsymbol{T} = v_j * t_{ijkl} \f$.
+
+  \param v vector \f$ \boldsymbol{v} \f$
+  \param ts  \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{T} \f$
+
+  \return \f$ \boldsymbol{v} | \boldsymbol{T} \f$
+*/
+
 //-------------------------------------------------------------------------------------
 template <class Tf> tensor3<Tf> operator | (const vector<Tf>& v, const tensor4<Tf>& ts)
 //-------------------------------------------------------------------------------------
@@ -530,6 +861,18 @@ template <class Tf> tensor3<Tf> operator | (const vector<Tf>& v, const tensor4<T
 }
 
 
+/*!
+  \brief Multiplication for \f$ 4^\text{th} \f$ order tensor and vector
+
+  Computes the multiplication of the tensor \f$ \boldsymbol{T} = (t_{ijkl}) \f$ with the vector \f$ \boldsymbol{v} = v_i \f$ for a given component \f$ n \f$ of the tensor.
+
+  \param ts  \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{T} \f$
+  \param v vector \f$ \boldsymbol{v} \f$
+  \param i component n
+
+  \return \f$ \boldsymbol{v} * \boldsymbol{T} \f$ (if n=1), \f$ \boldsymbol{v} | \boldsymbol{T} \f$ (if n=2), \f$ \boldsymbol{T} | \boldsymbol{v} \f$ (if n=3), \f$ \boldsymbol{T} * \boldsymbol{v} \f$ (if n=4)
+*/
+
 //--------------------------------------------------------------------------------------------
 template <class Tf> tensor3<Tf> tensor_sum (const tensor4<Tf>& ts, const vector<Tf>& v, int i)
 //--------------------------------------------------------------------------------------------
@@ -546,59 +889,113 @@ template <class Tf> tensor3<Tf> tensor_sum (const tensor4<Tf>& ts, const vector<
 }
 
 
-//----------------------------------------------------------------------------------------
-template <class Tf> tensor2<Tf> operator || (const tensor4<Tf>& t, const tensor2<Tf>& mat)
-//----------------------------------------------------------------------------------------
+//=====Public methods for tensor4 and tensor2==================================
+
+
+/*!
+  \brief Contracted product for \f$ 4^\text{th} \f$ order tensor with \f$ 2^\text{nd} \f$ order tensor
+
+  Computes the contracted product of the tensors \f$ \boldsymbol{a} = (a_{ijkl}) \f$ and \f$ \boldsymbol{b} = (b_{kl}) \f$ : \n
+  \f$ c_{ij} = a_{ijkl} b_{kl} \f$, where \f$ \boldsymbol{c} = (c_{ij}) \f$.
+
+  \param t \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{a} \f$
+  \param t2 \f$ 2^\text{nd} \f$ order tensor \f$ \boldsymbol{b} \f$
+
+  \return \f$ \boldsymbol{c} = \boldsymbol{a} :: \boldsymbol{b} \f$
+*/
+
+//---------------------------------------------------------------------------------------
+template <class Tf> tensor2<Tf> operator || (const tensor4<Tf>& t, const tensor2<Tf>& t2)
+//---------------------------------------------------------------------------------------
 {
   assert (t.size);
-  assert (t.dim3() == mat.Rows());
-  assert (t.dim4() == mat.Columns());
+  assert (t.dim3() == t2.Rows());
+  assert (t.dim4() == t2.Columns());
 
-  tensor2<Tf> mat_result(t.dim1(),t.dim2(),false);
+  tensor2<Tf> t2_result(t.dim1(),t.dim2(),false);
   for (int i=1; i<=t.dim1(); i++)
     for (int j=1; j<=t.dim2(); j++)
-      mat_result(i,j) = t[i][j] | mat;
+      t2_result(i,j) = t[i][j] | t2;
 
-  return mat_result;
+  return t2_result;
 }
 
 
-//----------------------------------------------------------------------------------------
-template <class Tf> tensor2<Tf> operator || (const tensor2<Tf>& mat, const tensor4<Tf>& t)
-//----------------------------------------------------------------------------------------
+/*!
+  \brief Contracted product for \f$ 2^\text{nd} \f$ order tensor with \f$ 4^\text{th} \f$ order tensor
+
+  Computes the contracted product of the tensors \f$ \boldsymbol{a} = (a_{ijkl}) \f$ and \f$ \boldsymbol{b} = (b_{ij}) \f$ : \n
+  \f$ c_{kl} = b_{ij} a_{ijkl} \f$, where \f$ \boldsymbol{c} = (c_{kl}) \f$.
+
+  \param t2 \f$ 2^\text{nd} \f$ order tensor \f$ \boldsymbol{b} \f$
+  \param t \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{a} \f$
+
+  \return \f$ \boldsymbol{c} = \boldsymbol{b} :: \boldsymbol{a} \f$
+*/
+
+//---------------------------------------------------------------------------------------
+template <class Tf> tensor2<Tf> operator || (const tensor2<Tf>& t2, const tensor4<Tf>& t)
+//---------------------------------------------------------------------------------------
 {
   assert (t.size);
-  assert (t.dim1() == mat.Rows());
-  assert (t.dim2() == mat.Columns());
+  assert (t.dim1() == t2.Rows());
+  assert (t.dim2() == t2.Columns());
 
-  tensor2<Tf> mat_result(t.dim3(),t.dim4());
+  tensor2<Tf> t2_result(t.dim3(),t.dim4());
   for (int i=1; i<=t.dim1(); i++)
     for (int j=1; j<=t.dim2(); j++)
       for (int k=1; k<=t.dim3(); k++)
 	for (int l=1; l<=t.dim4(); l++)
-	  mat_result(k,l) += t(i,j,k,l) * mat(i,j);
+	  t2_result(k,l) += t(i,j,k,l) * t2(i,j);
 
-  return mat_result;
+  return t2_result;
 }
 
 
-//-------------------------------------------------------------------------------------------
-template <class Tf> tensor4<Tf> operator ^ (const tensor2<Tf>& mat1, const tensor2<Tf>& mat2)
-//-------------------------------------------------------------------------------------------
-{
-  assert (mat1.Rows() && mat1.Columns() && mat2.Rows() && mat2.Columns());
+/*!
+  \brief Tensorial product for \f$ 2^\text{nd} \f$ order tensors
 
-  tensor4<Tf> tsr(mat1.Rows(),mat1.Columns(),mat2.Rows(),mat2.Columns(),false);
-  for (int i=1; i<=mat1.Rows(); i++)
-    for (int j=1; j<=mat1.Columns(); j++)
-      for (int k=1; k<=mat2.Rows(); k++)
-	for (int l=1; l<=mat2.Columns(); l++)
-	  tsr(i,j,k,l) = mat1(i,j) * mat2(k,l);
+  Computes the tensorial product for the tensors \f$ \boldsymbol{a} = (a_{ij}) \f$ and \f$ \boldsymbol{b} = (b_{kl}) \f$ : \n
+  \f$ c_{ijkl} = a_{ij} b_{kl} \f$, where \f$ \boldsymbol{c} = (c_{ijkl}) \f$.
+
+  \param t1 \f$ 2^\text{nd} \f$ order tensor \f$ \boldsymbol{a} \f$
+  \param t2 \f$ 2^\text{nd} \f$ order tensor \f$ \boldsymbol{b} \f$
+
+  \return \f$ \boldsymbol{c} = \boldsymbol{a} \times \boldsymbol{b} \f$
+*/
+
+//---------------------------------------------------------------------------------------
+template <class Tf> tensor4<Tf> operator ^ (const tensor2<Tf>& t1, const tensor2<Tf>& t2)
+//---------------------------------------------------------------------------------------
+{
+  assert (t1.Rows() && t1.Columns() && t2.Rows() && t2.Columns());
+
+  tensor4<Tf> tsr (t1.Rows(), t1.Columns(), t2.Rows(), t2.Columns(), false);
+  for (int i=1; i<=t1.Rows(); i++)
+    for (int j=1; j<=t1.Columns(); j++)
+      for (int k=1; k<=t2.Rows(); k++)
+	for (int l=1; l<=t2.Columns(); l++)
+	  tsr(i,j,k,l) = t1(i,j) * t2(k,l);
 
   return tsr;
 }
 
 
+//=====Public methods for tensor4 and tensor3==================================
+
+
+/*!
+  \brief Product for \f$ 3^\text{rd} \f$ order tensors
+
+  Computes the product of the tensors \f$ \boldsymbol{a} = (a_{n,ij}) \f$ and \f$ \boldsymbol{b} = (b_{n,kl}) \f$ : \n
+  \f$ c_{ijkl} = a_{n,ij} b_{n,kl} \f$, where \f$ \boldsymbol{c} = (c_{ijkl}) \f$.
+
+  \param t1 \f$ 3^\text{rd} \f$ order tensor \f$ \boldsymbol{a} \f$
+  \param t2 \f$ 3^\text{th} \f$ order tensor \f$ \boldsymbol{b} \f$
+
+  \return \f$ \boldsymbol{c} = \boldsymbol{a} \times \boldsymbol{b} \f$
+*/
+
 //---------------------------------------------------------------------------------------
 template <class Tf> tensor4<Tf> operator ^ (const tensor3<Tf>& t1, const tensor3<Tf>& t2)
 //---------------------------------------------------------------------------------------
@@ -612,6 +1009,18 @@ template <class Tf> tensor4<Tf> operator ^ (const tensor3<Tf>& t1, const tensor3
 }
 
 
+/*!
+  \brief Contracted product for \f$ 3^\text{rd} \f$ order tensor with \f$ 4^\text{th} \f$ order tensor
+
+  Computes the contracted product of the tensors \f$ \boldsymbol{a} = (a_{n,ij}) \f$ and \f$ \boldsymbol{b} = (b_{ijkl}) \f$ : \n
+  \f$ c_{n,kl} = a_{n,ij} b_{ijkl} \f$, where \f$ \boldsymbol{c} = (c_{n,kl}) \f$.
+
+  \param t3 \f$ 3^\text{rd} \f$ order tensor \f$ \boldsymbol{a} \f$
+  \param t4 \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{b} \f$
+
+  \return \f$ \boldsymbol{c} = \boldsymbol{a} :: \boldsymbol{b} \f$
+*/
+
 //----------------------------------------------------------------------------------------
 template <class Tf> tensor3<Tf> operator || (const tensor3<Tf>& t3, const tensor4<Tf>& t4)
 //----------------------------------------------------------------------------------------
@@ -629,6 +1038,18 @@ template <class Tf> tensor3<Tf> operator || (const tensor3<Tf>& t3, const tensor
 }
 
 
+/*!
+  \brief Contracted product for \f$ 4^\text{th} \f$ order tensor with \f$ 3^\text{rd} \f$ order tensor
+
+  Computes the contracted product of the tensors \f$ \boldsymbol{a} = (a_{ijkl}) \f$ and \f$ \boldsymbol{b} = (b_{n,kl}) \f$ : \n
+  \f$ c_{n,ij} = a_{ijkl} b_{n,kl} \f$, where \f$ \boldsymbol{c} = (c_{n,ij}) \f$.
+
+  \param t4 \f$ 4^\text{th} \f$ order tensor \f$ \boldsymbol{a} \f$
+  \param t3 \f$ 3^\text{rd} \f$ order tensor \f$ \boldsymbol{b} \f$
+
+  \return \f$ \boldsymbol{c} = \boldsymbol{a} :: \boldsymbol{b} \f$
+*/
+
 //----------------------------------------------------------------------------------------
 template <class Tf> tensor3<Tf> operator || (const tensor4<Tf>& t4, const tensor3<Tf>& t3)
 //----------------------------------------------------------------------------------------
@@ -646,6 +1067,17 @@ template <class Tf> tensor3<Tf> operator || (const tensor4<Tf>& t4, const tensor
 }
 
 
+//=====Public methods for tensor4==============================================
+
+
+/*!
+  \brief Computes symmetric \f$ 4^\text{th} \f$ order identity tensor
+
+  \param n space dimension
+
+  \return Identity tensor
+*/
+
 //------------------------------------------
 template <class Tf> tensor4<Tf> Id4s (int n)
 //------------------------------------------
@@ -677,7 +1109,7 @@ template <class Tf> tensor4<Tf> change_basis (const matrix<Tf>& P, const tensor4
 //----------------------------------------------------------------------------------------
 {
   assert ((P.Rows() ==3) && (P.Columns() == 3));
-  assert (tsr.size == 3 && tsr.dim2()==3 && tsr.dim3() && tsr.dim4()==3);
+  assert (tsr.size == 3 && tsr.dim2()==3 && tsr.dim3()==3 && tsr.dim4()==3);
 
   tensor4<Tf> Tsr(3);
   for (int p=1; p<=3; p++)
