@@ -2,7 +2,7 @@
  *                 Mathematical Object Library                 *
  *           class vector : declaration for vectors            *
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 2.2.0	               *
+ *	             GNU/linux version 2.4.0	               *
  *            software under General Public License            *
  ***************************************************************
  * copyright © 1994,1995,1996,1999,2000,2001,2002,2003,2004,2005,2006,2007,2008,2009 COLLARD Christophe
@@ -50,8 +50,8 @@
 	     copyright \htmlonly &#169; \endhtmlonly 2003, 2004 Denis MERCIER \n
 	     copyright \htmlonly &#169; 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009 Laboratoire de Physique et M&#233;canique des Mat&#233;riaux (LPMM - CNRS) \endhtmlonly \n
 	     copyright \htmlonly &#169; 2002, 2003, 2004, 2005, 2006, 2007 Laboratoire de Math&#233;matiques et ses Applications de Valenciennes (LAMAV) \endhtmlonly
-    \version 2.2.0
-    \date 1994-2009
+    \version 2.4.0
+    \date 1994-2010
     \bug none
     \warning none
 */
@@ -138,14 +138,14 @@ template <class T> class vector
     template <class Tf> friend vector<Tf> operator - (const vector<Tf>&, const vector<Tf>&); // overloads operator - for vectors
     template <class Tf> friend Tf         operator * (const vector<Tf>&, const vector<Tf>&); // defines the scalar product
     template <class Tf> friend vector<Tf> operator / (const vector<Tf>&, const vector<Tf>&); // exists only if b.size=1
-    void operator += (const vector<T>&);
-    void operator -= (const vector<T>&);
+    vector<T>& operator += (const vector<T>&);
+    vector<T>& operator -= (const vector<T>&);
 // operations between scalars and vectors
     template <class Tf> friend vector<Tf> operator * (const vector<Tf>&, const Tf&); // overloads vector * scalar operator
     template <class Tf> friend vector<Tf> operator * (const Tf&, const vector<Tf>&); // overloads scalar * vector operator
     template <class Tf> friend vector<Tf> operator / (const vector<Tf>&, const Tf&); // overloads vectors / scalars operator
-    void operator *=  (const T&);
-    void operator /=  (const T&);
+    vector<T>& operator *=  (const T&);
+    vector<T>& operator /=  (const T&);
 // overload for the outstream
     template <class Tf> friend ostream& operator << (ostream&, const vector<Tf>&);
     template <class Tf> friend istream& operator >> (istream&, const vector<Tf>&);
@@ -275,7 +275,7 @@ template <class T> void vector<T>::assign (int n)
 
 
 /*!
-  \brief Returns the address of a vector coordinate
+  \brief Returns the reference of a vector coordinate
 
   Warning : we use the mathematical notations so the vector coordinates are in the set [1;n], where n is the vector size.
   \param i vector coordinate
@@ -284,6 +284,8 @@ template <class T> void vector<T>::assign (int n)
   \f$ \textbf{vector$<$double$>$ v(5);} \; \textit{Allocates memory for a vector with type double and with 5 coordinates.} \f$ \n
   \f$ \textbf{v[2] = 1.245;} \; \textit{Stores 1.245 for the 2nd vector coordinate.} \f$ \n
   \f$ \textbf{cout $<<$ v[2];} \; \textit{Prints out the 2nd vector coordinate.} \f$ \n
+
+  \return \f$ i^\text{th} \f$ coordinate of the vector
 */
 
 //--------------------------------------------------------
@@ -323,7 +325,7 @@ template <class T> int vector<T>::operator = (int n)
 
   If both vectors are fully defined (size + table of coordinates), it checks that they have the same size and copies the coordinates. If the left hand side vector is partially defined (the size of the vector is zero) then it allocates memory for this vector and copies the coordinates. \n
   \param v vector to copy
-  \return reference of the left hand size vector (for multiple equalities)
+  \return reference of the left hand side vector (for multiple equalities)
   \n\n
   \f$ \textbf{Examples :} \f$ \n
   \f$ \textbf{vector$<$float$>$ v, w(4), z(4);} \; \textit{Defines 3 vectors with type float and allocates memory for "w" and "z" with 4 coordinates initialized to zero.} \f$ \n
@@ -348,10 +350,10 @@ template <class T> vector<T>& vector<T>::operator = (const vector<T>& v)
 /*!
   \brief Destructive operator = for vectors
 
-  Makes a fast copy of vector v parameters (size, address of the coordinates table "p") and deletes v (turns its size to zero). Note that we do not make any duplication of the coordinates table here. \n
+  Makes a fast copy of vector v data (size, address of the coordinates table "p") and deletes v (turns its size to zero). Note that we do not make any duplication of the coordinates table here. \n
   This operator is generally used with temporary objects.
   \param v vector address
-  \return reference of the left hand size vector (for multiple equalities)
+  \return reference of the left hand side vector (for multiple equalities)
   \n\n
   \f$ \textbf{Examples :} \f$ \n
   \f$ \textbf{vector$<$float$>$ v, w(4), z(4);} \; \textit{Defines 3 vectors with type float and allocates memory for "w" and "z" with 4 coordinates initialized to zero.} \f$ \n
@@ -383,14 +385,14 @@ template <class T> vector<T>& vector<T>::operator = (vector<T>* v)
 
 
 /*!
-  \brief Operator = for vectors with different sizes
+  \brief Operator = for vectors with different size
 
   If the size of the right hand side vector is zero, then we delete the table of coordinates of the left hand side vector. \n
   If both vectors are fully defined (size + table of coordinates) and have the same size then we only copy the coordinates. \n
-  If both vectors are fully defined but they don't have the same size, then we delete the table of coordinates of the left hand side vector and we reallocate memory for this table with the size of the right hand side vector, then we copy the coordinates. \n
+  If both vectors are fully defined but don't have the same size, then we delete the table of coordinates of the left hand side vector and we reallocate memory for this table with the size of the right hand side vector, then we copy the coordinates. \n
 
   \param v vector to copy
-  \return reference of the left hand size vector (for multiple equalities)
+  \return reference of the left hand side vector (for multiple equalities)
   \n\n
   \f$ \textbf{Examples :} \f$ \n
   \f$ \textbf{vector$<$float$>$ v, y, w(2), z(4);} \; \textit{Defines 4 vectors with type float and allocates memory for "w" and "z" coordinates.} \f$ \n
@@ -424,10 +426,10 @@ template <class T> vector<T>& vector<T>::operator &= (const vector<T>& v)
 /*!
   \brief Destructive operator = for vectors with different size
 
-  Makes a fast copy of vector v parameters (size, address of the coordinates table "p") and deletes v (turns its size to zero). Note that we do not make any duplication of the coordinates table here. \n
+  Makes a fast copy of the vector v data (size, address of the coordinates table "p") and deletes v (turns its size to zero). Note that we do not make any duplication of the coordinates table here. \n
   This operator is generally used with temporary objects.
   \param v vector address
-  \return reference of the left hand size vector (for multiple equalities)
+  \return reference of the left hand side vector (for multiple equalities)
   \n\n
   \f$ \textbf{Examples :} \f$ \n
   \f$ \textbf{vector$<$float$>$ v, w(2), z(4);} \; \textit{Defines 3 vectors with type float and allocates memory for "w" and "z" coordinates.} \f$ \n
@@ -519,6 +521,7 @@ template<class Tf> bool operator != (const vector<Tf>& a, const vector<Tf>& b)
   \f$ c_i = a_i + b_i \f$, where \f$ \boldsymbol{c} = (c_i) \f$.
   \param vector a
   \param vector b
+
   \return \f$ c = a+b \f$
   \n\n
   \f$ \textbf{Example :} \f$ \n
@@ -534,9 +537,10 @@ template <class Tf> vector<Tf> operator + (const vector<Tf>& a, const vector<Tf>
 {
   assert (a.size);
   assert (a.size == b.size);
+
   vector<Tf> v = a;
-  v += b;
-  return v;
+
+  return v += b;
 }
 
 
@@ -562,9 +566,10 @@ template <class Tf> vector<Tf> operator - (const vector<Tf>& a, const vector<Tf>
 {
   assert (a.size);
   assert (a.size == b.size);
+
   vector<Tf> v = a;
-  v -= b;
-  return v;
+
+  return v -= b;
 }
 
 
@@ -591,9 +596,11 @@ template <class Tf> Tf operator * (const vector<Tf>& a, const vector<Tf>& b)
 {
   assert (a.size);
   assert (a.size == b.size);
+
   Tf sp=0;
   for (int i=0; i<a.size; i++)
        sp += a.p[i] * b.p[i];
+
   return sp;
 }
 
@@ -622,30 +629,70 @@ template <class Tf> vector<Tf> operator / (const vector<Tf>& a, const vector<Tf>
   assert (a.size && (b.size==1)); // exists only if the size of b = 1
   assert (b.p[0]);
   vector<Tf> v = a;
-  v /= b.p[0];
-  return v;
+
+  return v /= b.p[0];
 }
 
 
-//-----------------------------------------------------------------
-template <class T> void vector<T>::operator += (const vector<T>& v)
-//-----------------------------------------------------------------
+/*!
+  \brief Add a vector.
+
+  Adds the vector \f$ \boldsymbol{v} = (v_i) \f$ to the current vector \f$ \boldsymbol{a} = (a_i) \f$ :
+  \f$ a_i += v_i \f$  (which is equivalent to \f$ a_i = a_i + v_i \f$ but faster).
+
+  \param vector v
+
+  \return \f$  \boldsymbol{a} =  \boldsymbol{a} +  \boldsymbol{v} \f$
+  \n\n
+  \f$ \textbf{Example :} \f$ \n
+  \f$ \textbf{vector$<$double$>$ w(4), z(4);} \; \textit{Defines 2 vectors with type double and allocates memory for "w" and "z" coordinates.} \f$ \n
+  \f$ \textbf{z[1]=1.23; z[2]=-3.4; z[4]=2.34;} \; \textit{Stores the coordinates of "z" (see} \f$ operator[](int)const \f$\textit{).} \f$ \n
+  \f$ \textbf{w[1]=-1.45; w[3]=0.436; w[4]=5.75;} \; \textit{Stores the coordinates of "w" (see} \f$ operator[](int)const \f$\textit{).} \f$ \n
+  \f$ \textbf{w += z;} \; \textit{Adds to "w" coordinates the ones of "z"} \textit{).} \f$ \n
+*/
+
+//-----------------------------------------------------------------------
+template <class T> vector<T>& vector<T>::operator += (const vector<T>& v)
+//-----------------------------------------------------------------------
 {
   assert (size);
   assert (size==v.size);
+
   for (int i=0; i<size; i++)
     p[i] += v.p[i];
+
+  return *this;
 }
 
 
-//-----------------------------------------------------------------
-template <class T> void vector<T>::operator -= (const vector<T>& v)
-//-----------------------------------------------------------------
+/*!
+  \brief Subtract a vector.
+
+  Subtracts the vector \f$ \boldsymbol{v} = (v_i) \f$ to the current vector \f$ \boldsymbol{a} = (a_i) \f$ :
+  \f$ a_i -= v_i \f$  (which is equivalent to \f$ a_i = a_i - v_i \f$ but faster).
+
+  \param vector v
+
+  \return \f$  \boldsymbol{a} =  \boldsymbol{a} -  \boldsymbol{v} \f$
+  \n\n
+  \f$ \textbf{Example :} \f$ \n
+  \f$ \textbf{vector$<$double$>$ w(4), z(4);} \; \textit{Defines 2 vectors with type double and allocates memory for "w" and "z" coordinates.} \f$ \n
+  \f$ \textbf{z[1]=1.23; z[2]=-3.4; z[4]=2.34;} \; \textit{Stores the coordinates of "z" (see} \f$ operator[](int)const \f$\textit{).} \f$ \n
+  \f$ \textbf{w[1]=-1.45; w[3]=0.436; w[4]=5.75;} \; \textit{Stores the coordinates of "w" (see} \f$ operator[](int)const \f$\textit{).} \f$ \n
+  \f$ \textbf{w -= z;} \; \textit{Subtracts to "w" coordinates the ones of "z"} \textit{).} \f$ \n
+*/
+
+//-----------------------------------------------------------------------
+template <class T> vector<T>& vector<T>::operator -= (const vector<T>& v)
+//-----------------------------------------------------------------------
 {
   assert (size);
   assert (size==v.size);
+
   for (int i=0; i<size; i++)
     p[i] -= v.p[i];
+
+  return *this;
 }
 
 
@@ -655,8 +702,8 @@ template <class Tf> vector<Tf> operator * (const vector<Tf>& v, const Tf& s)
 {
   assert (v.size);
   vector<Tf> w = v;
-  w *= s;
-  return w;
+
+  return w *= s;
 }
 
 
@@ -664,7 +711,7 @@ template <class Tf> vector<Tf> operator * (const vector<Tf>& v, const Tf& s)
 template <class Tf> vector<Tf> operator * (const Tf& s, const vector<Tf>& v)
 //--------------------------------------------------------------------------
 {
-  return v*s;
+  return v * s;
 }
 
 
@@ -680,23 +727,29 @@ template <class Tf> vector<Tf> operator / (const vector<Tf>& v, const Tf& s)
 }
 
 
-//---------------------------------------------------------
-template <class T> void vector<T>::operator *= (const T& s)
-//---------------------------------------------------------
+//---------------------------------------------------------------
+template <class T> vector<T>& vector<T>::operator *= (const T& s)
+//---------------------------------------------------------------
 {
   assert (size);
+
   for (int i=0; i<size; i++)
     p[i] *= s;
+
+  return *this;
 }
 
 
-//---------------------------------------------------------
-template <class T> void vector<T>::operator /= (const T& s)
-//---------------------------------------------------------
+//---------------------------------------------------------------
+template <class T> vector<T>& vector<T>::operator /= (const T& s)
+//---------------------------------------------------------------
 {
   assert (size);
+
   for (int i=0; i<size; i++)
     p[i] /= s;
+
+  return *this;
 }
 
 
@@ -705,9 +758,11 @@ template <class Tf> ostream& operator << (ostream& s, const vector<Tf>& v)
 //------------------------------------------------------------------------
 {
   assert (v.size);
+
   for (int i=0; i<v.size; i++)
     s << (v.p)[i] << " ";
   s << endl;
+
   return s;
 }
 
@@ -717,8 +772,10 @@ template <class Tf> istream& operator >> (istream& s, const vector<Tf>& v)
 //------------------------------------------------------------------------
 {
   assert (v.size);
+
   for (int i=0; i<v.size; i++)
       s >> (v.p)[i];
+
   return s;
 }
 
@@ -814,10 +871,12 @@ template <class T> long double vector<T>::norm () const
 //-----------------------------------------------------
 {
   assert (size);
-  long double n = p[0]*p[0];
+  long double sqr_norm = pow (p[0], 2);
+
   for (int i=1; i<size; i++)
-      n += p[i]*p[i];
-  return sqrt(n);
+    sqr_norm += pow (p[i], 2);
+
+  return sqrt (sqr_norm);
 }
 
 
@@ -826,10 +885,12 @@ template <class T> T vector<T>::norminf () const
 //----------------------------------------------
 {
   assert (size);
-  T n = abs(p[0]);
+  T norm = abs(p[0]);
+
   for (int i=1; i<size; i++)
-      if (abs(p[i]) > n) n = abs(p[i]);
-  return n;
+      if (abs(p[i]) > norm) norm = abs(p[i]);
+
+  return norm;
 }
 
 
@@ -839,8 +900,11 @@ template <class T> vector<T>& vector<T>::approximation ()
 {
   assert (size);
   T elt;
+  T norm = sup ((T)(*this).norm(), (T) 1);
+
   for (int i=0; i<size; i++)
-    if (abs(p[i]) < epsilon) p[i]=0;
+    if (abs(p[i]) < epsilon * norm) p[i]=0;
+
   return *this;
 }
 
