@@ -1,6 +1,6 @@
 /***************************************************************
  *                    simula+@metz.ensam.fr                    *
- *                   GNU/linux version 2.2.0                   *
+ *                   GNU/linux version 2.4.0                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright © 2003,2004,2005,2006,2007,2008,2009 COLLARD Christophe
@@ -360,6 +360,10 @@ int test_matrix (int detail)
   //  if (detail) affiche ("inversion gauss pivot total", (matx*gauss_pt(matx) == matxId) && (a*gauss_pt(a) == aId) && (matxx*gauss_pt(matxx) == matxxId));
   //  else result *= ((matx*gauss_pt(matx) == matxId) && (a*gauss_pt(a) == aId) && (matxx*gauss_pt(matxx) == matxxId));
 
+  if (detail) affiche ("p-norm (with p=2)", abs (sqrt (tr (t(nca) * nca)) - nca.p_norm(2)) < epsilon);
+  else result *= (abs (sqrt (tr (t(nca) * nca)) - nca.p_norm(2)) < epsilon);
+
+
   matrix<long double> Id_2(5,5,true,0);
   for (int i=1; i<=5; i++)
     Id_2(i,i) = 1;
