@@ -1,6 +1,6 @@
 /***************************************************************
  *                    simula+@metz.ensam.fr                    *
- *                   GNU/linux version 2.2.0                   *
+ *                   GNU/linux version 2.4.0                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright © 2003,2004,2005,2006,2009 COLLARD Christophe
@@ -100,6 +100,10 @@
 #include "tensors4-test.h"
 #endif
 
+#if !defined(__SYMTENSORS4_TEST_H)
+#include "symtensors4-test.h"
+#endif
+
 #if !defined(__CONVERSION_TOOLS_TEST_H)
 #include "conversion tools-test.h"
 #endif
@@ -227,7 +231,7 @@ main()
   start = clock();
   if (detail)
     //=======================================================================
-    cout << "----------------2nd order symmetric tensors---------------- \n";
+    cout << "----------------symmetric 2nd order tensors---------------- \n";
     //=======================================================================
   result *= test_symtensor2(detail);
   end = clock();
@@ -259,6 +263,18 @@ main()
 
   cout << endl;
 
+  detail = 0;
+  start = clock();
+  if (detail>0)
+    //=======================================================================
+    cout << "----------------symmetric 4th order tensor----------------- \n";
+    //=======================================================================
+  result *= test_symtensor4(detail);
+  end = clock();
+  cout<<"cpu time : " << (long double)(end-start)/CLOCKS_PER_SEC << endl;
+
+  cout << endl;
+
   detail = 0;
   start = clock();
   if (detail)
@@ -397,17 +413,6 @@ main()
 
   end_main = clock();
 
-  bool test = true;
-  ifstream file("../simula+.log", ios::in);
-  if (!file.fail())
-    file >> test;
-  file.close();
-  test *= result;
-  ofstream file2("../simula+.log", ios::out);
-  assert (!file2.fail());
-  file2 << test;
-  file2.close();
-
   cout << "============================================================== \n";
   if (result) cout << green << "-------OK-------------end of MOL++ tests-------------OK-------" << reset;
   else cout << red << "-----FAILED-----------end of MOL++ tests-----------FAILED-----" << reset;
